#include "mainpart.hpp"
#include "gamestrings.hpp"
#include "levels.hpp"
#include <math.h>

void MainPart::reset()
{
    ending=false;
    endingtimer=0;
    platformon[0]=0;
    platformon[1]=0;
    for (unsigned int i=0;i<entities.size();i++)
    {
        delete entities[i];
    }
    if (player[0]!=NULL)
        delete player[0];
    if (player[1]!=NULL)
        delete player[1];
    entities.clear();
    hearts.clear();
    platforms.clear();
    spikes.clear();
    loadmap(global_variables->level);
    for (int i=0;i<2;i++)
        for (int j=0;j<4;j++)
            moving[i][j]=false;
    soundengine->play(5);
}


enum TileType {tt_block,tt_start1,tt_start2 ,tt_platform1,tt_platform2,tt_crumble,tt_spikes,tt_heart,tt_blank};

TileType tiletype(int tile)
{

    switch (tile)
    {
    case 0:
        return tt_block;
    case 1:
        return tt_block;
    case 2:
        return tt_block;
    case 3:
        return tt_block;
    case 4:
        return tt_block;
    case 5:
        return tt_block;
    case 6:
        return tt_spikes;
    case 7:
        return tt_spikes;
    case 8:
        return tt_platform1;
    case 9:
        return tt_platform2;
    case 10:
        return tt_heart;
    case 11:
        return tt_heart;
    case 12:
        return tt_spikes;
    case 13:
        return tt_spikes;
    case 14:
        return tt_start1;
    case 15:
        return tt_start2;
    case 16:
        return tt_blank;
    case 17:
        return tt_blank;
    case 18:
        return tt_blank;
    case 19:
        return tt_crumble;
    case 20:
        return tt_crumble;
    case 21:
        return tt_crumble;
    case 22:
        return tt_crumble;
    case 23:
        return tt_crumble;
    case 24:
        return tt_blank;
    case 25:
        return tt_blank;
    case 26:
        return tt_blank;
    case 27:
        return tt_blank;
    case 28:
        return tt_blank;
    case 29:
        return tt_blank;
    case 30:
        return tt_blank;//SHOULD NEVER REACH THESE ONES; KISSY SPRITES
    case 31:
        return tt_blank;
    case 32:
        return tt_blank;
    case 33:
        return tt_blank;
    case 34:
        return tt_blank;
    case 35:
        return tt_blank;
    default:
        return tt_blank;
    };
};

void MainPart::loadmap(int i)
{
    glDeleteLists(maplist,1);
    maplist=glGenLists (1);

    glNewList(maplist, GL_COMPILE);//generate map as go along
    global_variables->heartcount=0;
    global_variables->maxhearts=0;

    for (int i=0;i<30;i++)
        for (int j=0;j<30;j++)
        {
            map.mask[29-j][i]=false;
            int newtile;

            switch (global_variables->level)
            {
                case 1:
                    newtile=level1[j][i]-1;
                    break;
                case 2:
                    newtile=level2[j][i]-1;
                    break;
                case 3:
                    newtile=level3[j][i]-1;
                    break;
                case 4:
                    newtile=level4[j][i]-1;
                    break;
                case 5:
                    newtile=level5[j][i]-1;
                    break;
                case 6:
                    newtile=level6[j][i]-1;
                    break;
            }

            TileType tt = tiletype(newtile);
//{tt_heart,tt_blank};

            switch (tt)
            {
            case tt_block:
                graphicsengine->tiles->drawTileIM(7*i,7*(30-j),newtile);
                map.mask[29-j][i]=true;
                break;
            case tt_start1:
                graphicsengine->tiles->drawTileIM(7*i,7*(30-j),newtile);
                player[0]=new Player(7*i+2,7*(30-j)-1,0,&moving[0][0],&map,soundengine);
                map.player[0]=player[0];
                break;
            case tt_start2:
                graphicsengine->tiles->drawTileIM(7*i,7*(30-j),newtile);
                player[1]=new Player(7*i+2,7*(30-j)-1,1,&moving[1][0],&map,soundengine);
                map.player[1]=player[1];
                break;
            case tt_platform1:
            {
                Platform* e = new Platform(7*i,7*(30-j),d_left,&map,soundengine);
                entities.push_back(e);
                platforms.push_back(e);
            }

            //FILL IN LATER
            break;
            case tt_platform2:
            {
                Platform* e = new Platform(7*i,7*(30-j),d_up,&map,soundengine);
                entities.push_back(e);
                platforms.push_back(e);
            }
            //FILL IN LATER
            break;
            case tt_crumble:
                //FILL IN LATER
                break;
            case tt_spikes:
            {
                graphicsengine->tiles->drawTileIM(7*i,7*(30-j),newtile);
                Spikes* e = new Spikes(7*i,7*(30-j),&map,newtile,soundengine);
                entities.push_back(e);
                spikes.push_back(e);
            }
            break;
            case tt_heart:
                Heart* h = new Heart(7*i,7*(30-j),&map,soundengine);
                hearts.push_back(h);
                entities.push_back(h);
                global_variables->maxhearts++;
                //FILL IN LATER
                break;
            case tt_blank:
                break;
            }

        }
    glEndList();

    float sx=(player[0]->x+player[1]->x)/2.0f;
    float sy=(player[0]->y+player[1]->y)/2.0f;
    float dx=abs(player[0]->x-player[1]->x);
    float dy=abs(player[0]->y-player[1]->y);
    translatex=(-16-14+30-sx);
    translatey=(-11+13-sy);
    scale=(0.0355841f+(0.0083704-0.0355841f)*sqrt(max(dx,dy)/(30.0f*7.0f) ));
    rotatex=(-20.0f+ sy*20.0f/203.0f  );

};

MainPart::MainPart(const SubengineInterface* subengines, Global_Variables* p_global_variables):
        EnginePart(subengines),
        maplist(0),
        rotatex(-15),
        rotatey(-15),
        rotatez(0),
        translatex(-101),
        translatey(-107),
        translatez(-1),
        scale(0.0093704),
        global_variables(p_global_variables)
{
    player[0]=NULL;
    player[1]=NULL;

    soundengine->streamtrack("audio/level6.mp3");
}

MainPart::~MainPart()
{
    for (unsigned int i=0;i<entities.size();i++)
    {
        delete entities[i];
    }
}

int MainPart::input(const SDL_Event& event)
{
    if ( event.type == SDL_KEYDOWN )
    {
        switch ( event.key.keysym.sym )
        {
        case SDLK_ESCAPE:
            return 3;
            break;
        case SDLK_SPACE:
        case SDLK_RETURN:
            break;
        case SDLK_UP:
            moving[1][0]=true;
            break;
        case SDLK_DOWN:
            moving[1][1]=true;
            break;
        case SDLK_LEFT:
            moving[1][2]=true;
            break;
        case SDLK_RIGHT:
            moving[1][3]=true;
            break;
        case SDLK_w:
            moving[0][0]=true;
            break;
        case SDLK_s:
            moving[0][1]=true;
            break;
        case SDLK_a:
            moving[0][2]=true;
            break;
        case SDLK_d:
            moving[0][3]=true;
            break;/*
        case SDLK_q:
            rotatex+=5;
            break;
        case SDLK_w:
            rotatey+=5;
            break;
        case SDLK_e:
            rotatez+=5;
            break;
        case SDLK_a:
            rotatex-=5;
            break;
        case SDLK_s:
            rotatey-=5;
            break;
        case SDLK_d:
            rotatez-=5;
            break;
        case SDLK_r:
            scale*=1.1f;
            break;
        case SDLK_f:
            scale/=1.1f;
            break;
        case SDLK_t:
            translatex+=1.0f;
            break;
        case SDLK_g:
            translatex-=1.0f;
            break;
        case SDLK_y:
            translatey+=1.0f;
            break;
        case SDLK_h:
            translatey-=1.0f;
            break;
        case SDLK_u:
            translatez+=1.0f;
            break;
        case SDLK_j:
            translatez-=1.0f;
            break;
        case SDLK_p:
            std::cout<<player[0]->x<<","<<player[0]->y<<","<<player[1]->x<<","<<player[1]->y<<endl;
            break;*/
        default:
            break;
        }
    }
    else
        if ( event.type == SDL_KEYUP )
        {
            switch ( event.key.keysym.sym )
            {
            case SDLK_UP:
                moving[1][0]=false;
                break;
            case SDLK_DOWN:
                moving[1][1]=false;
                break;
            case SDLK_LEFT:
                moving[1][2]=false;
                break;
            case SDLK_RIGHT:
                moving[1][3]=false;
                break;
            case SDLK_w:
                moving[0][0]=false;
                break;
            case SDLK_s:
                moving[0][1]=false;
                break;
            case SDLK_a:
                moving[0][2]=false;
                break;
            case SDLK_d:
                moving[0][3]=false;
                break;
            default:
                break;
            }
        }

    return 0;
}

static inline int relintersect(int l1,int t1,int r1,int b1,int l2,int t2,int r2,int b2)
{
    if (l1 > r2 || r1 < l2 ||
            t1 >= b2 || b1 <= t2)
        return (-1);

    //check if first is above second
    if (b1>b2 && b1>t2)
    {
        return 0;
    }
    //else check if below
    else if (b2>b1 && b2>t1)
    {
        return 1;
    }
    //else check if to left
    else if (r1>=l2 && r1<=r2)
    {
        return 2;
    }
    //else must be right
    else
        return 3;
}

static inline bool intersect(int l1,int t1,int r1,int b1,int l2,int t2,int r2,int b2)
{
    return (!(l1 > r2 || r1 < l2 ||
              t1 >= b2 || b1 <= t2));
}

int MainPart::logic()
{

//    -15,-15,0,0.0355841,-16,-11,2,30,13
    float sx=(player[0]->x+player[1]->x)/2.0f;
    float sy=(player[0]->y+player[1]->y)/2.0f;
    float dx=abs(player[0]->x-player[1]->x);
    float dy=abs(player[0]->y-player[1]->y);
    translatex+=((-16-14+30-sx)-translatex)/8.0f;
    translatey+=((-11+13-sy)-translatey)/8.0f;
    translatez=2;

    rotatex+=((-20.0f+ sy*20.0f/203.0f  )-rotatex)/8.0f;
    //rotatex is -17 when y=0, 0 when y=203
    scale+=((0.0355841f+(0.0083704-0.0355841f)*sqrt(max(dx,dy)/(30.0f*7.0f) ) )-scale)/8.0f;
    //target scale is 0.0355841 when max(dx,dy)=0, 0.0093704 when max(dx,dy)=30*7


    foreach(Platform* e,platforms)
    {
        e->move();
        e->carry(0);
        e->carry(1);
    }

    if (ending)
    {
        endingtimer++;
        if (endingtimer>120)
            return 1;
    }
    else
    {

        player[0]->move();
        player[1]->move();

        bool oldonplatform;
        for (int i=0;i<2;i++)
        {
            oldonplatform=player[i]->onplatform;
            player[i]->onplatform=false;
            foreach(Platform* e, platforms)
            {
                //            e->carry(1);
                switch (relintersect(e->x+1,
                                     e->y,
                                     e->x+e->width-1,
                                     e->y+e->height+2,
                                     player[i]->x,
                                     player[i]->y,
                                     player[i]->x+player[i]->width,
                                     player[i]->y+player[i]->height-2
                                    )
                       )
                {
                case -1:
                    break;
                case 0://above player
                    player[i]->y=e->y-e->height;
                    player[i]->playerstate=ps_falling;
                    player[i]->statecounter=0;
                    player[i]->tile=2*tilesperline*player[i]->num+tilesperline*player[i]->facingl+10;
                    break;
                case 1: //below player:

                    player[i]->onplatform=true;
                    platformon[i]=e;
                    if ((!oldonplatform) && player[i]->playerstate!=ps_crouching && player[i]->playerstate!=ps_walking)
                    {
                        player[i]->playerstate=ps_standing;
                        player[i]->statecounter=0;
                        player[i]->tile=2*tilesperline*player[i]->num+tilesperline*player[i]->facingl+2;
                    }
                    player[i]->y=e->y+player[i]->height;

                    break;
                case 2: //left of player:
                    player[i]->x=e->x+e->width;
                    break;
                case 3: //right of player:
                    player[i]->x=e->x-player[i]->width;
                    break;
                }


            }
        }

        foreach(Spikes* e, spikes)
        {
            if ( intersect(
                        e->x,
                        e->y,
                        e->x+e->width,
                        e->y+e->height,
                        player[0]->x+1,
                        player[0]->y,
                        player[0]->x+player[0]->width-2,
                        player[0]->y+player[0]->height-2
                    ))
            {

                player[0]->kill();
            }
            if ( intersect(
                        e->x,
                        e->y,
                        e->x+e->width,
                        e->y+e->height,
                        player[1]->x+1,
                        player[1]->y,
                        player[1]->x+player[1]->width-2,
                        player[1]->y+player[1]->height-2
                    ))
            {

                player[1]->kill();
            }
        }


        foreach(Heart* h,hearts)
        {

            if (
                (!h->collected)
                &&
                intersect(
                    h->x,
                    h->y,
                    h->x+h->width,
                    h->y+h->height,
                    player[0]->x+1,
                    player[0]->y,
                    player[0]->x+player[0]->width-2,
                    player[0]->y+player[0]->height-2
                )
                &&
                intersect(
                    h->x,
                    h->y,
                    h->x+h->width,
                    h->y+h->height,
                    player[1]->x+1,
                    player[1]->y,
                    player[1]->x+player[1]->width-2,
                    player[1]->y+player[1]->height-2
                ))
            {
                h->collected=true;
                h->tile=11;
                global_variables->heartcount++;
                soundengine->play(3);
                if (global_variables->heartcount==global_variables->maxhearts)
                {
                    ending=true;
                    //END LEVEL SOMEHOW
                }
            }
        }
    }

    return 0;
}

void MainPart::draw()
{

    glLoadIdentity();

    glRotatef(rotatex,1,0,0);
    glRotatef(rotatey,0,1,0);
    glRotatef(rotatez,0,0,1);

    glScalef(scale,scale,scale);
    //draw hearts


    glTranslatef(translatex,translatey,translatez);


    //DRAW MAP
    glCallList(maplist);

    if (ending)
    {
        graphicsengine->tiles->drawTile(player[0]->x,player[0]->y+2,30+min((endingtimer/20),5));

    }
    else
    {
        //DRAW PLAYERS
        graphicsengine->sprites->drawTile(player[0]->x,player[0]->y,player[0]->tile);
        graphicsengine->sprites->drawTile(player[1]->x,player[1]->y,player[1]->tile);

        //DRAW NPCS
        foreach (Entity* e,entities)
        {
            graphicsengine->tiles->drawTile(e->x,e->y,e->tile);
        }

    }


    glDisable(GL_DEPTH_TEST);
    glLoadIdentity();
    glScalef(0.0093704,0.0093704,0.0093704);
    float f=0;
    foreach(Heart* h, hearts)
    {
        graphicsengine->tiles->drawTile(-100+f,100,h->tile);
        f+=10;
    }

    glEnable(GL_DEPTH_TEST);


}
